@@ -11,6 +11,14 @@ _cache = cache.key(__name__)
 
 class PAPIError(BossmanError):
   pass
+class PAPIValidationError(PAPIError):
+  pass
+class PAPIRuleTreeValidationError(PAPIValidationError):
+  pass
+class PAPIHostnamesValidationError(PAPIValidationError):
+  pass
+class PAPIRuleFormatSchemaNotFoundError(BossmanError):
+  pass
 class PAPIVersionAlreadyActiveError(PAPIError):
   pass
 class PAPIVersionAlreadyActivatingError(PAPIError):
@@ -44,6 +52,23 @@ class PAPIPropertyVersion:
     self.productionStatus = kwargs.get("productionStatus")
     self.stagingStatus = kwargs.get("stagingStatus")
 
+class PAPIPropertyVersionHostnames:
+  def __init__(self, **kwargs):
+    self.accountId = kwargs.get("accountId")
+    self.contractId = kwargs.get("contractId")
+    self.groupId = kwargs.get("groupId")
+    self.propertyId = kwargs.get("propertyId")
+    self.propertyName = kwargs.get("propertyName")
+    self.propertyVersion = kwargs.get("propertyVersion")
+    self.etag = kwargs.get("etag")
+    self.hostnames = kwargs.get("hostnames")
+    self.errors = kwargs.get("errors", [])
+    self.warnings = kwargs.get("warnings", [])
+
+  @property
+  def has_errors(self):
+    return len(self.errors) > 0
+
 class PAPIPropertyVersionRuleTree:
   def __init__(self, **kwargs):
     self.accountId = kwargs.get("accountId")
@@ -136,9 +161,10 @@ class PAPIClient:
     self.logger.debug("get_property_id propertyName={propertyName}".format(propertyName=propertyName))
     property_id = None
     response = self.session.post("/papi/v1/search/find-by-value", json={"propertyName": propertyName})
-    versions = list(PAPIPropertyVersion(**item) for item in response.json().get("versions").get("items"))
-    if len(versions) > 0:
-      property_id = str(versions[0].propertyId)
+    if response.status_code == 200:
+      versions = list(PAPIPropertyVersion(**item) for item in response.json().get("versions", {}).get("items", []))
+      if len(versions) > 0:
+        property_id = str(versions[0].propertyId)
     return property_id
 
   def get_property(self, propertyName) -> PAPIProperty:
@@ -238,6 +264,8 @@ class PAPIClient:
     response = self.session.put(url, json=ruleTree, headers=headers)
     if response.status_code == 200:
       return PAPIPropertyVersionRuleTree(**response.json())
+    if response.status_code == 400:
+      raise PAPIRuleTreeValidationError(response.json())
     raise PAPIError(response.json())
 
   def update_property_hostnames(self, propertyId, version, hostnames):
@@ -245,7 +273,9 @@ class PAPIClient:
     url = "/papi/v1/properties/{propertyId}/versions/{version}/hostnames".format(propertyId=propertyId, version=version)
     response = self.session.put(url, json=hostnames)
     if response.status_code == 200:
-      return response.json()
+      return PAPIPropertyVersionHostnames(**response.json())
+    if response.status_code == 400:
+      raise PAPIHostnamesValidationError(response.json())
     raise PAPIError(response.json())
 
   @lru_cache(maxsize=1000) # don't fetch more than once per session, even if the rule format wasn't persistently cacheable
@@ -256,6 +286,10 @@ class PAPIClient:
     if schema is None:
       url = "/papi/v1/schemas/products/{productId}/{ruleFormat}".format(productId=productId, ruleFormat=ruleFormat)
       response = self.session.get(url)
+      if response.status_code == 404:
+        raise PAPIRuleFormatSchemaNotFoundError(response.json())
+      elif response.status_code != 200:
+        raise PAPIError(response.json())
       schema = response.json()
       if ruleFormat != "latest":
         cache.update_json(schema)
@@ -268,6 +302,8 @@ class PAPIClient:
     if schema is None:
       url = "/papi/v1/schemas/request/{request_filename}".format(request_filename=request_filename)
       response = self.session.get(url)
+      if response.status_code != 200:
+        raise PAPIError(response.json())
       schema = response.json()
       cache.update_json(schema)
     return schema
