@@ -9,6 +9,8 @@ import gitdb.exc
 import yaml
 import threading
 
+from bossman.rich import bracketize
+
 def true(*args, **kwargs):
   return True
 
@@ -254,7 +256,7 @@ class Revision:
 
   def __str__(self):
     with self.repo._lock:
-      s = "[{}] {} {} | {}".format(self.id, self.short_message, self.author_name, self.date)
+      s = "{} {} {} | {}".format(bracketize(self.id), self.short_message, self.author_name, self.date)
       if len(self.changes):
         s += "\n\n  "
         s += "\n  ".join(str(change) for change in self.changes.values())
@@ -263,7 +265,7 @@ class Revision:
 
   def __rich_console__(self, console, options):
     with self.repo._lock:
-      yield "[b]\[{}][/b] {} | {} {}".format(self.id, self.short_message, self.author_name, self.date)
+      yield "[bold]{}[/bold] {} | {} {}".format(bracketize(self.id), self.short_message, self.author_name, self.date)
       # for change in self.changes.values():
       #   yield change
 
@@ -461,17 +463,17 @@ class Repo:
     return self._repo.head.ref.name
 
   @synchronized
-  def get_branches(self) -> list:
-    return tuple(branch.name for branch in self._repo.branches)
-
-  @synchronized
-  def get_branches_containing(self, rev: str) -> list:
+  def get_branches(self, **kwargs) -> list:
     try:
-      result = self._repo.git.branch(contains=rev, format="%(refname:short)")
+      result = self._repo.git.branch(format="%(refname:short)", **kwargs)
       return result.splitlines()
     except git.GitCommandError:
       return []
 
+  @synchronized
+  def get_branches_containing(self, rev: str, **kwargs) -> list:
+    return self.get_branches(contains=rev, **kwargs)
+
   @synchronized
   def get_revisions(self, since_rev: str = None, until_rev: str = "HEAD",  paths: list = None) -> list:
     """
@@ -481,7 +483,19 @@ class Repo:
     commitRange = ("{since_rev}..{until_rev}".format(since_rev=since_rev, until_rev=until_rev)
       if since_rev
       else until_rev)
-    commits = self._repo.iter_commits(commitRange, paths=paths)
+
+    # Traversing history with merge support. If we have history like this:
+    #
+    # *  a051acd (HEAD -> main) Merge branch 'test' into main
+    # |\  
+    # | * 8303358 (test) cache css/jss for 30d
+    # | * 96a1826 cache errors for 74s
+    # |/  
+    # * 4bbe5f0 (origin/main) update README
+    #
+    # We will traverse directly from a051acd to 4bbe5f0. For bossman, this means
+    # that merge commits can effectively be used as release commits.
+    commits = self._repo.iter_commits(commitRange, paths=paths, first_parent=True)
     revisions = []
     for commit in commits:
       prev = git.NULL_TREE
